!-----------------------------------------------------------------------------
!
! Non-linear power spectrum of velocity divergence, theta=div(velocity),
! P_tt(k), computed using the 3rd order perturbation theory.
!
! P_tt(k) = P_11(k) + P_22(k) + 2P_13(k)
!
! where P_11(k) is the linear power spectrum, supplied by the user.
! Ref. e.g., Eq.(64), (67) and (69) of Scoccimarro, PRD, 70, 083007 (2004) 
!
! *** P(k) is in units of h^-3 Mpc^3, and k is in units of h Mpc^-1 ***
! (This is the same as CAMB definition.)
! 
! One can scale this result to an arbitrary redshift, z, by rescaling 
! the linear growth factor as
!
! P_tt(k,z) = [D(z)/D(z*)]^2 P_11(k) + [D(z)/D(z*)]^4 [P_22(k) + 2P_13(k)]
!
! where D(z)/D(z*) is the linear growth factor relative to the input linear
! power spectrum computed at z=z*.
!
! << sample code >>
!
! USE linearpk
! double precision :: k_ov_h,p11,p22,p13
! character(len=128) :: filename
! integer :: n
! filename='wmap5baosn_max_likelihood_matterpower.dat'
! n=896 ! # of lines in the file
! CALL open_linearpk(filename,n)
! k_ov_h=0.01d0 ! h Mpc^-1
! CALL pkt(k_ov_h,p11,p22,p13)
! print*,'P_11(k) at k=',k_ov_h,' h Mpc^-1 is',p11,' h^-3 Mpc^3'
! print*,'P_22(k) at k=',k_ov_h,' h Mpc^-1 is',p22,' h^-3 Mpc^3'
! print*,'P_13(k) at k=',k_ov_h,' h Mpc^-1 is',p13,' h^-3 Mpc^3'
! print*,'** Total power spectrum is P_11 + P_22 + 2*P_13. **'
! CALL close_linearpk
! end
!
! August 23, 2008
! E. Komatsu
!
!-----------------------------------------------------------------------------
SUBROUTINE PkT(ak,p11,p22,p13)
  IMPLICIT none
  DOUBLE PRECISION, parameter :: lnkmin = -9.2 ! kmin=1e-4 h Mpc^-1
  DOUBLE PRECISION, intent(OUT) :: ak,p11,p22,p13
  DOUBLE PRECISION :: ss1,ss2,ss3,ss4,eps,lnkmax,p13_t
  DOUBLE PRECISION :: dp13dq_t,dp22dq_t,linear_pk
  DOUBLE PRECISION, parameter :: pi=3.14159265358979d0
  EXTERNAL dp13dq_t,dp22dq_t
  !!$eps = ak*1.d-2
  eps = ak*0.17
  lnkmax = 3.d0
  !!$ lnkmax = 4.d0 ! made very little differnce. lnkmax=3 is good enough.
  if(dlog(eps)<lnkmin)then
     print*,'k is too small: must be greater than',exp(lnkmin)/0.17
     stop
  endif
!$ tree level PT
  p11 = linear_pk(ak)
!!$ P_t_t(k)
  !!$ one-loop PT: P_13(k)
  CALL qromb(dp13dq_t,lnkmin,lnkmax,ss1,ak)
  p13_t = p11*ss1/(2.*pi)**2.
  !!$ one-loop PT: P_22(k): use trick of Jain*Bertschinger(1994) (Eq.[22])
  CALL qromb1(dp22dq_t,lnkmin,dlog(eps),ss1,ak,0.d0)
  CALL qromb1(dp22dq_t,dlog(eps),dlog(ak-eps),ss2,ak,0.d0)
  CALL qromb1(dp22dq_t,dlog(ak-eps),dlog(ak+eps),ss3,ak,eps)
  CALL qromb1(dp22dq_t,dlog(ak+eps),lnkmax,ss4,ak,0.d0)
  p22=2.*(2.*ss1+ss2+ss3+ss4)/(2.*pi)**2.
  p13=0.5d0*p13_t
  return
END SUBROUTINE PkT
!------------------------------------------------
DOUBLE PRECISION FUNCTION dp13dq_t(lnq,k)
  IMPLICIT none
  DOUBLE PRECISION :: k,lnq,q
  DOUBLE PRECISION :: G3,linear_pk
  q = dexp(lnq)
  dp13dq_t = q*G3(k,q)*linear_pk(q)
  return
END FUNCTION dp13dq_t
!------------------------------------------------
DOUBLE PRECISION FUNCTION dp22dq_t(lnq,k,eps)
  IMPLICIT none
  DOUBLE PRECISION :: k,lnq,q,eps,lim
  DOUBLE PRECISION :: dGGdmu,ss,linear_pk
  EXTERNAL dGGdmu
  q = dexp(lnq)
  if (eps.eq.0.) then
     CALL qromb2(dGGdmu,-1.d0,1.d0,ss,q,k)
  else
     lim = (k**2+q**2-eps**2)/(2.*k*q)
     CALL qromb2(dGGdmu,-1.d0,lim,ss,q,k)
  endif
  dp22dq_t = q**3.*linear_pk(q)*ss
  return
END FUNCTION dp22dq_t
!------------------------------------------------
DOUBLE PRECISION FUNCTION dGGdmu(mu,q,k)
  IMPLICIT none
  DOUBLE PRECISION :: k,p,q,mu
  DOUBLE PRECISION :: G2,linear_pk
  p = sqrt(k**2.+q**2.-2.*k*q*mu)
  dGGdmu = G2(k,q,mu)**2.*linear_pk(p)
  return
END FUNCTION dGGdmu
!---------------------------------------
DOUBLE PRECISION FUNCTION G2(k,q,mu)
  IMPLICIT none
!!$ vec{p} = vec{k} - vec{q}
  DOUBLE PRECISION :: k,q,mu
  DOUBLE PRECISION :: p,kp,kq,pq
  kq = k*q*mu
  kp = k**2.-kq
  pq = kq-q**2 
  p  = sqrt(k**2+q**2-2*kq)
  G2 = 3./7. &
      + (1./2.)*pq/(p*q)*( (p/q)+(q/p) ) &
      + (4./7.)*pq**2./(p*q)**2.
  return
END FUNCTION G2
!---------------------------------------
DOUBLE PRECISION FUNCTION G3(k,q)
  IMPLICIT none
  DOUBLE PRECISION  k,q
!!$ Ref: Eq.[2.28] of Makino, Sasaki and Suto, PRD, 46, 585 (1992)
!!$ (Their equation needs to be multiplied by 9k^2/4.)
  if(k/q>1d-1)then
     G3 = k**2./84.*( &
          12.*(k/q)**2 - 82. + 4.*(q/k)**2 - 6*(q/k)**4 &
          + (3./k**5/q**3)*(q**2-k**2)**3*(q**2+2.*k**2.) &
            *dlog((k+q)/abs(k-q)))
  else
     G3 = k**2.*(-6./5.+104./245.*k**2/q**2) ! expansion around k/q=0
  endif
  return
END FUNCTION G3 
